﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Monkeyspeak
{
	[Serializable]
	public class VariableIsConstantException : Exception
	{
		public VariableIsConstantException() { }

		public VariableIsConstantException(string message) : base(message) { }

		public VariableIsConstantException(string message, Exception inner) : base(message, inner) { }

		protected VariableIsConstantException(
		  System.Runtime.Serialization.SerializationInfo info,
		  System.Runtime.Serialization.StreamingContext context)
			: base(info, context) { }
	}

	[Serializable]
	public class Variable
	{
		public static readonly Variable NoValue = new Variable("%none", "", false);

		private bool isConstant;

		public bool IsConstant
		{
			get
			{
				return isConstant;
			}
			set
			{
				isConstant = value;
			}
		}

		private object vvalue;

		public object Value
		{
			get
			{
				return vvalue;
			}
			  set
			{
                 // removed AssignValue as it interfered with page.setVariable - Gerolkae
				vvalue = value;
			}
		}

		private string name;

		public string Name
		{
			get
			{
				return name;
			}
			internal set
			{
				name = value;
			}
		}

        // Variable var = new variable(); 
        // Preset reader.readvariable with default data
        // Needed for Conditions  checking Variables that haven't been defined yet.
        // -Gerolkae
        internal Variable()
        {
            isConstant = false;
            name = "%none";
            vvalue = "";
        }

		internal Variable(string Name, object value, bool constant = false)
		{
			isConstant = constant;
			name = Name;
			vvalue = value;
		}

		/// <summary>
		/// Attempts to assign a value to this variable
		/// <para>Throws VariableIsConstantException if the variable is a constant.</para>
		/// </summary>
		/// <param name="value"></param>
		public void AssignValue(object value)
		{
			if (CheckType(value) == false) throw new TypeNotSupportedException(value.GetType().Name +
" is not a supported type. Expecting string or double.");

			if (IsConstant == false)
				vvalue = value;
				
			else throw new VariableIsConstantException("Attempt to assign a value to constant \"" + Name + "\"");
		}

		public void ForceAssignValue(object value)
		{
			if (CheckType(value) == false) throw new TypeNotSupportedException(value.GetType().Name +
" is not a supported type. Expecting string or double.");
			vvalue = value;
		}

		private bool CheckType(object value)
		{
			if (value == null) return true;

			if (value is String ||
				value is Double)
				return true;
			return false;
		}

		/// <summary>
		/// Returns a const identifier if the variable is constant followed by name,
		/// <para>otherwise just the name is returned.</para>
		/// </summary>
		/// <returns></returns>
		public override string ToString()
		{
			return ((IsConstant) ? "const " : "") + Name + " = " + ((Value == null) ? "null" : Value.ToString());
		}

		/// <summary>
		///
		/// </summary>
		/// <param name="asConstant">Clone as Constant</param>
		/// <returns></returns>
		public Variable Clone(bool asConstant = false)
		{
			return new Variable(this.Name, this.Value, asConstant);
		}

		public static bool operator ==(Variable varA, Variable varB)
		{
			return varA.Value == varB.Value;
		}

		public static bool operator !=(Variable varA, Variable varB)
		{
			return varA.Value != varB.Value;
		}

		public override bool Equals(object obj)
		{
            return ((Variable)obj).Name.Equals(this.Name) && ((Variable)obj).Value.Equals(this.Value);
		}
	}
}